/* volume.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef VOLUME_H
#define VOLUME_H

// This is a proxy to VolumeBase.  As such, it encapsulates any of a
// number of different kinds of volumes transparently.  See the
// comments in volumeBase.h.

#include "referenceCount.h"

#include <string>

class Dataset;
class TableDataProxy;
class VolumeBase;

class Volume : public ReferenceCountProxy {
public:
  Volume();
  Volume(int volume_id);
  Volume(const Dataset &data, int volume_id);
  Volume(const Dataset &data, const TableDataProxy &volume);

  bool is_valid() const;

  string get_description() const;
  string get_definition() const;
  int get_volume_id() const;

  bool exists() const;
  bool can_read_files() const;
  bool can_store_files() const;

  // get_filename() returns the filename in the Unix directory of the
  // indicated file within this volume, iff the volume is mounted and
  // readable through normal Unix system calls.  Otherwise, it returns
  // the empty string.
  string get_filename(const string &rel_filename);

  // prepare_write_file() returns the full pathname in the Unix
  // directory of the indicated file-to-be within this volume, in the
  // bucket it probably will be stored in.  However, this might change
  // if we later discover we have overfilled the bucket.  Once the
  // file is written and closed, call close_write_file().
  string prepare_write_file(const string &basename);
  
  // close_write_file() is called after writing a file into the
  // volume.  It checks the bucket size, and may move the file if the
  // bucket is filled.  It returns the relative pathname of the file
  // within the volume.
  string close_write_file(const string &full_pathname,
			  const string &basename);

  // store_file() takes a file which already exists in the Unix
  // filesystem and moves it into the volume, wherever that may be.
  // It returns the new filename on success, empty string on failure.
  string store_file(const string &full_pathname, 
		    const string &basename, bool leave_orig);
  
  // Re-fetch the parameters from the database to make sure we're
  // totally in sync with all other processes.
  void read_db();

protected:
  VolumeBase *get_ptr() const;
};

#endif
